#include "KNNPredictor.h"
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
using namespace std;

int tests_passed = 0;
int points = 0;

void addPoints(int test_number) {
	switch (test_number) {
	case 1:
		points += 1;
		break;
	case 2:
		points += 1;
		break;
	case 3:
		points += 2;
		break;
	case 4:
		points += 3;
		break;
	case 5:
		points += 2;
		break;
	case 6:
		points += 2;
		break;
	case 7:
		points += 3;
		break;
	case 8:
		points += 3;
		break;
	case 9:
		points += 1;
		break;
	case 10:
		points += 1;
		break;
	case 11:
		points += 1;
		break;
	case 12:
		points += 2;
		break;
	case 13:
		points += 2;
		break;
	case 14:
		points += 5;
		break;
	case 15:
		points += 1;
		break;
	case 16:
		points += 5;
		break;
	}
}

// Funkcija koja poredi Vas izlaz sa ocekivanim izlazom za odgovarajuci test primer
void checkOutput(const string& log_filename, int test_number) {

	ifstream correct_output("correct_log_" + to_string(test_number) + ".txt");
	stringstream ss_correct;
	ss_correct << correct_output.rdbuf(); correct_output.close();
	string correct_file_system = ss_correct.str();

	ifstream student_output(log_filename);
	stringstream ss_student;
	ss_student << student_output.rdbuf(); student_output.close();
	string student_file_system = ss_student.str();

	if (student_file_system != "" && student_file_system == correct_file_system) {
		cout << "Test " + to_string(test_number) + ": PASSED" << endl;
		addPoints(test_number);
		tests_passed++;
	}
	else {
		cout << "Test " + to_string(test_number) + ": FAILED" << endl;
	}

}

void test(int test_number) {

	fstream file("student_log_" + to_string(test_number) + ".txt", ios::out);

	Vector vector_1;
	vector_1.addValue(5.0);
	vector_1.addValue(3.0);
	vector_1.addValue(-4.0);

	Vector vector_2;
	vector_2.addValue(2.5);
	vector_2.addValue(-1.76);
	vector_2.addValue(0);

	Vector vector_3;
	vector_3.addValue(2.5);

	Vector vector_4;
	vector_4.addValue(3.0);
	vector_4.addValue(4.0);

	Vector vector_5;
	vector_5.addValue(0.0);
	vector_5.addValue(0.0);

	Vector vector_6;
	vector_6.addValue(4.0);
	vector_6.addValue(2.0);

	DistanceCalculator* calc = nullptr;

	VectorSpace space;

	KNNPredictor* predictor = nullptr;

	switch (test_number) {
	case 1:
		file << "Broj elemenata: " << vector_1.getSize();
		break;
	case 2:
		file << "Prvi element: " << fixed << setprecision(1) << vector_1[0] << endl;
		file << "Drugi element: " << fixed << setprecision(1) << vector_1[1] << endl;
		file << "Treci element: " << fixed << setprecision(1) << vector_1[2];
		break;
	case 3:
		file << "Suma svih elemenata: " << fixed << setprecision(1) << vector_1.sum();
		break;
	case 4:
		file << "Prvi element razlike: " << (vector_1 - vector_2)[0];
		break;
	case 5:
		file << "Prvi element proizvoda: " << (vector_1 * vector_2)[0];
		break;
	case 6:
		file << "Apsolutna vrednost prvog elementa: " << fixed << setprecision(1) << vector_1.abs()[0] << endl;
		file << "Apsolutna vrednost drugog elementa: " << fixed << setprecision(1) << vector_1.abs()[1] << endl;
		file << "Apsolutna vrednost treceg elementa: " << fixed << setprecision(1) << vector_1.abs()[2];
		break;
	case 7:
		file << "Velicina podvektora: " << vector_1.subvector(0, 1).getSize();
		break;
	case 8:
		file << "Vektor: " << vector_1;
		break;
	case 9:
		file << "Vektor: " << vector_2;
		break;
	case 10:
		try {
			vector_1[3];
			file << "...";
		}
		catch (exception&) {
			file << "Greska!";
		}
		break;
	case 11:
		try {
			Vector v = vector_1 - vector_3;
			file << "...";
		}
		catch (exception&) {
			file << "Greska!";
		}
		break;
	case 12:
		calc = new EuclideanDistanceCalculator();
		file << "Udaljenost: " << fixed << setprecision(1) << calc->distance(vector_4, vector_5);
		delete calc;
		break;
	case 13:
		calc = new ManhattanDistanceCalculator();
		file << "Udaljenost: " << fixed << setprecision(1) << calc->distance(vector_4, vector_5);
		delete calc;
		break;
	case 14:
		space.read("input_14.txt");
		file << space;
		break;
	case 15:
		space.read("input_15.txt");
		file << "Dimenzija prostora: " << space.dimension();
		break;
	case 16:
		space.read("input_16.txt");
		calc = new ManhattanDistanceCalculator();
		predictor = new KNNPredictor(space, calc, 3);
		file << "Predikcija: " << fixed << setprecision(1) << predictor->predict(vector_6);
		delete calc;
		delete predictor;
		break;
	}

	file.close();

	checkOutput("student_log_" + to_string(test_number) + ".txt", test_number);
}

int main() {

	// Izmeniti elemente niza u zavisnosti od testova koje zelite da pokrenete!
	int active_tests[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 };
	int number_of_tests = sizeof(active_tests) / sizeof(int);

	cout << "============================================" << endl;
	cout << "TESTING FILE_SYSTEM" << endl << endl;
	cout << "Executing tests: ";
	for (int i = 0; i < number_of_tests; i++) cout << active_tests[i] << (i < number_of_tests - 1 ? ", " : "");
	cout << endl;

	for (int i = 0; i < number_of_tests; i++) test(active_tests[i]);

	cout << "============================================" << endl;
	cout << "Tests passed count: " << tests_passed << endl;
	cout << "Points: " << points << endl;

	return 0;
}